# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

__all__ = ["ChatCompletionRole"]

ChatCompletionRole = Literal["system", "user", "assistant", "tool", "function"]
